//+------------------------------------------------------------------+
//|                                                     MetaSage.mq5 |
//|                                 Copyright 2011, Vladimir Gomonov |
//|                                            MetaDriver@rambler.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011, Vladimir Gomonov"
#property link      "MetaDriver@rambler.ru"
#property version   "1.00"
#property stacksize 65520
#property script_show_inputs

input uint SMax=99; //  
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
struct sMX
  {
   uint              count;
   uint              mx[256];
                     sMX() {count=0;}
   void Add(sMX &a) //  
     {for(uint i=0;i<a.count;i++) mx[i+count]=a.mx[i]; count+=a.count;}
   void Join(sMX &a,sMX &b) {count=0;Add(a);Add(b);}
   uint CalcFirst(uint in) //    ,    
     {
      count++;
      for(uint i=2; i<=sqrt(in);i++) if(in%i==0) return mx[0]=i;
      return mx[0]=in;
     }
   uint GetCountAllSums() //      
     {return(int) MathRound(MathPow(2,count-1))-1; }
   uint Value(){uint p=1;for(uint i=0;i<count;i++) p*=mx[i];return p;}
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CMults
  {
   sMX               MX[];  //    
public:
   uint              Top;   //    
                     CMults()
     {
      //        MMax-1 
      if((bool)-ArrayResize(MX,SMax*(SMax-1)))
        {Print(" ArrayResize(MX,SMax*(SMax-1))); ErrorCode=",GetLastError());}
      for(uint i=2;i<(uint)ArraySize(MX);i++)
        {
         uint temp=MX[i].CalcFirst(i); //  
         if(temp==i) continue;           //    
         MX[i].Add(MX[i/temp]);        //     
        }
      //    
      for(Top=SMax/2;MX[Top].count>1;Top++);
      Top+=2;
     }

   void PrintOne(uint i) //  .        [2..MMax-1]
     {
      string S=IntegerToString(MX[i].mx[0]);
      for(uint ii=1;ii<MX[i].count;ii++) S+=" * "+IntegerToString(MX[i].mx[ii]);
      Print(IntegerToString(i)," == ",S);
     }

   uint GetCountValidSum(uint n,uint &P,uint &a,uint &b)
     {
      uint Count=0;
      for(uint i=2;i<=sqrt(n);i++)
        {
         uint count=0;
         sMX J;
         J.Join(MX[i],MX[n-i]); //   
         for(uint j=1; j<=J.GetCountAllSums(); j++)
            count+=IsValidSum(J,j); // j -  
         if(count==1)
           {
            Count++;
            P=J.Value();
            a=i;
            b=n-i;
           }
        }
      return Count;
     }

   uint IsValidSum(sMX &J,uint n)
     {
      uint a=1,b=1;
      //     
      //  , ,   
      for(uint i=0; i<J.count;i++)
        {
         if((bool)((1<<i)  &n)) a*=J.mx[i];
         else b*=J.mx[i];
        }
      return(uint)ValidSum(a+b);
     }

   bool ValidSum(uint n) {return((n%2==1) && (MX[n-2].count>1) && n<=Top);}

  };
//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
  {
   CMults *M=new CMults;
   uint P,a,b;
   Print("//==============  ========================");
   Print("//+----   = ",SMax," -------------------+");
   for(uint i=4;i<=M.Top;i++)
      if(M.ValidSum(i))
         if(M.GetCountValidSum(i,P,a,b)==1)
           Print("S=",i,"; P=",P,"; a=",a,"; b=",b);
   Print("//+----   = ",SMax," -------------------+");
   delete M;
  }
//+------------------------------------------------------------------+
